/**************************************************************************************
Author:         David Szybala
Website:       www.davidszybala.com
Alt Website:  www.twistedgerbil.com
Program:       reduce_keys.ms
Date:           07/15/11
Description:   This script will reduce the amount of animation keys
***************************************************************************************/

try (destroyDialog reduce_keys_rollout) catch()


(--put script into local scope
    global reduce_keys_rollout
    local reduce_keys_ini_file = getDir #plugcfg + "\\ReduceKeys.ini"
    local reduce_keys_rollout_pos
    
    /**********************************************************************
    UI creation
    **********************************************************************/
    rollout reduce_keys_rollout "Reduce Keys"
    (
        groupBox controller_group "Controllers" pos:[5,5] width:175 height:80
            checkbox position_checkbox "Position" pos:[15,25] width:60 height:15 checked:true
            checkbox rotation_checkbox "Rotation" pos:[15,45] width:60 height:15 checked:true
            checkbox scale_checkbox "Scale" pos:[15,65] width:60 height:15 checked:true
        
        groupBox reduce_group "Reduce" pos:[5,90] width:175 height:80
            label threshold_label "Threshold:" pos:[15,110] width:55 height:15
			spinner threshold_spinner "" pos:[70,110] range:[0.01,100,0.1] type:#float fieldwidth:35 height:15
			label sample_label "Samples:" pos:[15,130] width:55 height:15
			spinner sample_spinner "" pos:[70,130] range:[0.01,9999,1] type:#float fieldwidth:35 height:15
			label frame_label "frame" pos:[120,130] width:30 height:15
            label range_label "Range:" pos:[15,150] width:45 height:15
            spinner range_start_spinner "" pos:[60,150] width:50 height:15 range:[-9999,9998,0] type:#integer
			label to_label " - " pos:[110,150] width:10 height:15
            spinner range_end_spinner "" pos:[120,150] width:50 height:15 range:[-9999,9998,100] type:#integer

        groupBox option_group "Options" pos:[5,175] width:175 height:40
            checkbox selection_only_checkbox "Selected Objects Only" pos:[15,195] width:125 height:15 checked:true
            
        button reduce_key_button "Reduce Keys" pos:[5,225] width:175 height:20
        label status_label "INFO: Standing by..." pos:[5,255] width:175 height:15
        
        /**********************************************************************
        UI open event handler. Prepares the UI for first time open
        **********************************************************************/
        on reduce_keys_rollout open do
        (
            range_start_spinner.value = (animationRange.start.frame as integer)
            range_end_spinner.value = (animationRange.end.frame as integer)
        ) --end on
        
        /**********************************************************************
        UI move event handler. Stores the curent position of the UI
        **********************************************************************/
        on reduce_keys_rollout moved pos do
            setIniSetting reduce_keys_ini_file "Dialog" "Position" (pos as string)
        
        /**********************************************************************
        Start range changed event handler. Ensures the start range is not greater than the end range
        **********************************************************************/
        on range_start_spinner changed the_value do
		(
			if the_value > range_end_spinner.value do
                range_end_spinner.value = the_value
		)--end on
		
        /**********************************************************************
        End range changed event handler. Ensures the end range is not less than the start range
        **********************************************************************/
		on range_end_spinner changed the_value do
		(
			if the_value < range_start_spinner.value do
                range_start_spinner.value = the_value
		)--end on
        
        /**********************************************************************
        Reduce button clicked event handler. Reduces the animation keys for selected/all objects
        **********************************************************************/
        on reduce_key_button pressed do
        (
            local reduce_objects = #()
            
            if not position_checkbox.checked and not rotation_checkbox.checked and not scale_checkbox.checked do
            (
                status_label.text = "ERROR: No controller selected"
                return false
            ) --end if
            
            if range_start_spinner.value == range_end_spinner.value do
            (
                status_label.text = "ERROR: Range is 0"
                return false
            ) --end if
            
            if selection_only_checkbox.checked then
                reduce_objects = selection as array
            else
                reduce_objects = objects
            
            for current in reduce_objects do
            (
                if position_checkbox.checked do
                (
                    if (isProperty current "position") do
                        reduceKeys current.position.controller threshold_spinner.value sample_spinner.value (interval range_start_spinner.value range_end_spinner.value)
                ) --end if
                
                if rotation_checkbox.checked do
                (
                    if (isProperty current "rotation") do
                        reduceKeys current.rotation.controller threshold_spinner.value sample_spinner.value (interval range_start_spinner.value range_end_spinner.value)
                ) --end if
                
                if scale_checkbox.checked do
                (
                    if (isProperty current "scale") do
                        reduceKeys current.scale.controller threshold_spinner.value sample_spinner.value (interval range_start_spinner.value range_end_spinner.value)
                ) --end if
            )--end for
            
            status_label.text = "INFO: Reduce complete"
            
            --CRITICAL! reduceKeys opens the listener window and closes this rollout.
            --No clue why. This is a hacky workaround
            showDialog reduce_keys_rollout true
            try
                uiaccessor.closedialog (windows.getchildhwnd 0 "MAXScript Listener")[1]
            catch()
        )--end on
    )--end rollout
    
    reduce_keys_rollout_pos = execute (getIniSetting reduce_keys_ini_file "Dialog" "Position")
    if reduce_keys_rollout_pos == OK do reduce_keys_rollout_pos = [100,100]
    if reduce_keys_rollout_pos.x > sysinfo.desktopsize.x - 185 do
        reduce_keys_rollout_pos.x = sysinfo.desktopsize.x - 190
    if reduce_keys_rollout_pos.y > sysinfo.desktopsize.y - 330 do
        reduce_keys_rollout_pos.y = sysinfo.desktopsize.y - 330
    if reduce_keys_rollout_pos.x < 0 do reduce_keys_rollout_pos.x = 0 
    if reduce_keys_rollout_pos.y < 0 do reduce_keys_rollout_pos.y = 0
    createDialog reduce_keys_rollout 185 270 reduce_keys_rollout_pos.x reduce_keys_rollout_pos.y
)--end local scope
